@@ -13,7 +13,7 @@ use 5.010001;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.878';
+our $VERSION = '1.879';
 
 use MCE::Shared::Base ();
 use MCE::Util ();
@@ -299,15 +299,15 @@ sub timedwait {
    $_timeout = 0.0003 if $_timeout < 0.0003;
 
    local $@; eval {
-      local $SIG{ALRM} = sub { die "alarm clock restart\n" };
+      local $SIG{ALRM} = sub { alarm 0; die "alarm clock restart\n" };
       alarm $_timeout unless $_is_MSWin32;
 
       die "alarm clock restart\n"
          if $_is_MSWin32 && MCE::Util::_sock_ready($_CV->{_cr_sock}, $_timeout);
 
-      MCE::Util::_sysread($_CV->{_cr_sock}, my($_b), 1);
-
-      alarm 0 unless $_is_MSWin32;
+      (!$_is_MSWin32)
+         ? (MCE::Util::_sysread($_CV->{_cr_sock}, my($_b1), 1), alarm(0))
+         : (MCE::Util::_sysread($_CV->{_cr_sock}, my($_b2), 1));
    };
 
    alarm 0 unless $_is_MSWin32;
@@ -354,7 +354,7 @@ MCE::Shared::Condvar - Condvar helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Condvar version 1.878
+This document describes MCE::Shared::Condvar version 1.879
 
 =head1 DESCRIPTION
 
